/*
	OpenGL model (CGLModel) is a container for objects derived from CGLObject.
	
*/

enum GLOB_TYPE { GLOB_TEXT, GLOB_BLOCK, GLOB_POLYCYL, GLOB_RIBBON };


class CGLVector : public CItem { 
	public : 
		CGLVector( GLfloat dX=0.0, GLfloat dY=0.0, GLfloat dZ=0.0, COLORREF cr = RGB(0,0,0) );
		void Set( GLfloat dX, GLfloat dY, GLfloat dZ, COLORREF cr );
		GLfloat m_dX;
		GLfloat m_dY;
		GLfloat m_dZ;
		COLORREF m_cr;
	};

//-- base GL object - contains origin, rotation, scaling and methods to move axis
class CGLObject : public CItem { 
	public : 
		GLfloat m_dX, m_dY, m_dZ;
		GLfloat m_dXRot, m_dYRot, m_dZRot;
		GLfloat m_dXScale, m_dYScale, m_dZScale;
		bool m_bVisible, m_bColor;
		GLOB_TYPE m_iType;
		
	public : 
		virtual ~CGLObject() { } 
		void SetBase( GLOB_TYPE iType, GLfloat dX, GLfloat dY, GLfloat dZ,
						GLfloat dXRot, GLfloat dYRot, GLfloat dZRot,
						GLfloat dXScale, GLfloat dYScale, GLfloat dZScale ) ;
		void Prepare();	
		virtual void Render(  );
		void RenderColor( COLORREF cr ); 
		void BlendColor( COLORREF cr0, COLORREF cr1, double dFrac );

	};
		
//-- Text 
class CGLText : public CGLObject {
	public : 
		char * m_pchText;
		COLORREF m_cr;
		CGLText();
		~CGLText();
		void Set( COLORREF cr, const char * pchText );
		void Render(  );
	};		

//-- Prism with 6 colored faces 
class CGLBlock : public CGLObject { 
	public : 
		COLORREF m_cr[6];	// front , back, left, right, top, bottom
		COLORREF m_crText;
		void Render(  );
		void SetColors( COLORREF crZ0, COLORREF crZ1, COLORREF crX0, COLORREF crX1, COLORREF crY0, COLORREF crY1 );
		void SetText( COLORREF cr, const char * pchText );
		CGLBlock();
		~CGLBlock();
		
		bool m_bWire;			// mainly for debugging purposes, draw solid as wireframe
	protected : 
		char * m_pchText;
		void Render2Quads( int iCR, GLfloat dx0, GLfloat dy0, GLfloat dz0,
							 GLfloat dx1, GLfloat dy1, GLfloat dz1 );
	};

//-- pseudo cylinder of i Sides
class CGLPolyCyl : public CGLObject { 
	public :	
		COLORREF m_cr0, m_cr1, m_crBody;
		void Render( );
		void Set( int iFaces, COLORREF cr0, COLORREF cr1, COLORREF crBody ) ;
	protected : 
		int m_iFaces;			// 2..48
		double * paX, *paY;
		void Calculate();
	};
	
//-- Cosine 'Ribbon' along x axis from 0 to m_dLen curving through dYm at midpoint and dY1 at endpoint.
class CGLRibbon : public CGLObject { 
	public : 
		GLfloat m_dYm, m_dY1;		// mid pt. height and end point.
		GLfloat m_dLen; 
		GLfloat m_dWidthFactor;		// [0,1.0] amount of distance between states to be 'plateau'
		COLORREF m_cr0, m_crM, m_cr1;
		bool m_bFine;
		void Set( int iFaces, bool bEndCaps, 
				  GLfloat dX0, GLfloat dY0, GLfloat dZ0,
				  GLfloat dYm, 
				  GLfloat dX1, GLfloat dY1, GLfloat dZ1, 
				  GLfloat dWidthFactor,
				  COLORREF cr0, COLORREF crM, COLORREF cr1 ) ;
		void Render( );
	protected : 
		int m_iFaces;			// 2..48
		bool m_bEndCaps;		// whether to draw endcaps
	};
	
	
//-- Collection of GL objects
class CGLModel {
	public :
		//GLYPHMETRICSFLOAT m_agmf[GL_TEXTMAX-GL_TEXTMIN+1];	// character display lists 
		bool m_bFontSetup;
		void GLFontSetup( HDC hdc );
		
		CGLModel() { 
			m_bFontSetup=false; 
			}

		CItemArray m_aObj;
		void Render( );
		void AddObject( CGLObject * pObj );
		void SetColor( bool bColor );
		CGLObject * GetObject( int iObj );
		
		void SetFine( bool bFine );
	};
		
		
/*
//----- Legend
  if (cbChecked=chkLegend.State) and ( cbChecked=chkColor.State) Then Begin
  	glLoadIdentity;           // Un-rotate
    glListBase(GL_TEXTBASE-GL_TEXTMIN);
    glColor3f(0.0,0.0,0.0);

    c := FloatToStr(m_nMinZ); // Minimum (text)
    glPushMatrix();
    glTranslate(110.0, -105.0, 0.0 );
    glScalef(6.0, 6.0, 6.0 );
    glCallLists(Length(c), GL_UNSIGNED_BYTE, PChar(c));
    glPopMatrix();

    c := FloatToStr(m_nMaxZ); // Maximum (text)
    glPushMatrix();
    glTranslate(110.0, 100.0, 0.0 );
    glScalef(6.0, 6.0, 6.0 );
    glCallLists(3, GL_UNSIGNED_BYTE, PChar(c));
    glPopMatrix();

    glLoadIdentity;           // Color bar at right side
  	glBegin(GL_TRIANGLE_STRIP);
    for i := 0 to 100 do begin
      n := (1.0*i) / 100.0;
    	glColor3f( 1.0-n, 0.25, n );
      glVertex3f(120.0, 200.0*n-100.0, 0.0 );
      glVertex3f(125.0, 200.0*n-100.0, 0.0 );
    end;
  end;
*/